package Grenouille::Queue;

# This software may be used and distributed according to the terms
# of the GNU General Public License, incorporated herein by reference.

use strict;
use Exporter qw();
use Time::HiRes qw(time);
use Grenouille::Vars qw($QUEUEFILE $ASAP $SEC $MIN %vars);
use Grenouille::Numbers qw(digits jitter);
use Grenouille::Log qw(display_and_log);
use Grenouille::Scheduler qw(schedule_event);
use Grenouille::Comm qw(contact_server $SERVERNAME);
use vars qw(@ISA @EXPORT_OK @result_queue);
@ISA = qw(Exporter);
@EXPORT_OK = qw(init_result_queue load_result_queue queue_result
                send_results);

# --------------------------- RESULTS QUEUING ----------------------------

@result_queue = ();

# dumping result queue to disk allow it to survive program reloads,
# but this is not strictly required for correct working
my($queue_storage_ok) = 1;

# Could we "tie" the queue structure to avoid explicit storage instructions ?

sub store_result_queue() {
    if($queue_storage_ok) {
	if(open(QUEUE, ">$QUEUEFILE")) {
	    map { print QUEUE "$_->{'time'}\n$_->{'string'}\n~\n" } 
              @result_queue;
	    close QUEUE;
	} else {
	    display_and_log('WARNING', "Impossible de mettre  jour $QUEUEFILE, critures dsactives");
	    $queue_storage_ok = 0;
	}
    }
}

sub load_result_queue() {
    if(open(QUEUE, "<$QUEUEFILE")) {
	while(not eof(QUEUE)) {
	    my($time, $string, $sep);
	    chomp($time = <QUEUE>);
	    chomp($string = <QUEUE>);
	    chomp($sep = <QUEUE>); # $sep is for consistency checking
	    push @result_queue, { 'time' => $time,
				  'string' => $string } if $sep eq '~';
	}
    }
}

sub init_result_queue() {
    @result_queue = ();
}

sub send_results($) { # EVENT
    my($schedule_time) = shift;
    return unless $vars{'id'};
    return unless scalar @result_queue;
    
    my ($result) = $result_queue[0];
    my ($result_age) = digits((time() - $result->{'time'}) / $SEC, 0);
    eval {
        contact_server("$result->{'string'}&id=$vars{'id'}&elapsed_seconds=$result_age");
    };
    if($@) {
	if($@ =~ /^Server responded error/) {
	    display_and_log('ERROR', $@);
	    if ($@ =~ /Identifiant d.utilisateur incorrect/) {
		display_and_log('INFO', "session expire, ouverture d'une nouvelle session");
                delete $vars{'id'};
		schedule_event($ASAP, \&open_session);
	    } else {
		# unidentified (possibly serious) problem encountered, 
		# drop the result to avoid looping
		display_and_log('WARNING', "type d'erreur non reconnu, annulation de l'envoi");
		shift @result_queue;
		store_result_queue();
	    }
	} else {
	    display_and_log('INFO', "Problme temporaire de connexion avec le serveur $SERVERNAME: $@");
	}
	schedule_event(time() + jitter(5 * $MIN, -1, 1), \&send_results);
    } else {
	shift @result_queue;
	store_result_queue();
	schedule_event(time() + 5 * $SEC, \&send_results);
    }
}

sub queue_result($) {
    my ($result) = { 'string' => $_[0], 'time' => time() };
    push @result_queue, $result;
    store_result_queue();
    schedule_event($ASAP, \&send_results);
}

1;
