package Grenouille::Scheduler;

# This software may be used and distributed according to the terms
# of the GNU General Public License, incorporated herein by reference.

use strict;
use Exporter qw();
use Time::HiRes qw(time sleep);
use Grenouille::Vars qw(%DEBUG $ASAP $SEC);
use Grenouille::Log qw(display_and_log);
use vars qw(@ISA @EXPORT_OK
            @events);
@ISA=qw(Exporter);
@EXPORT_OK=qw(clear_all_events get_next_event schedule_event);

# --------------------------- SCHEDULER ----------------------------
# events are scheduled "optimistically" by other events, so
# it's up to each event handler to check, when scheduled,
# if it can really run.
#
# a given event handler cannot be scheduled more than once 
# (the "soonish" schedule wins).

@events = ();

sub clear_all_events() {
    @events = ();
}

sub get_next_event() {
    die 'No event to wait for!' unless scalar @events;
    my($nextevent) = shift @events;
    my($wait) = ($nextevent->{'time'} - time()) / $SEC;
    if ($wait > 0) {
	print "(pause de $wait secondes...)" if $DEBUG{'scheduler'};
	sleep($wait);
	print "\n" if $DEBUG{'scheduler'};
    }
    return $nextevent;
}

sub schedule_event($$) {
    my($new_event) = { 'time' => $_[0], 'action' => $_[1] };
    my($now) = time();
    if($_[0] < $now) {
	display_and_log('WARNING', 'We cannot schedule an event in the past...') unless $_[0] == $ASAP;
	$new_event->{'time'} = $now;
    }
    
  INSERT: {
      my($i, @nevents);
      for($i = 0; $i < scalar @events;$i++) {
	  last if $new_event->{'time'} < $events[$i]->{'time'};
	  last INSERT if $events[$i]->{'action'} == $new_event->{'action'};
	  push @nevents, $events[$i];
      }
      push @nevents, $new_event;
      for(; $i < scalar @events;$i++) {
	  push @nevents, $events[$i] unless $events[$i]->{'action'} == 
                                            $new_event->{'action'};
      }
      @events = @nevents;
  }
}

1;
